/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** 
 * @brief Blinky Sample Application main file.
 *
 * This file contains the source code for a sample server application using the LED Button service.
 */

#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_conn_params.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "app_button.h"
#include "ble_lbs.h"
#include "bsp.h"
#include "ble_gap.h"
#include "SEGGER_RTT.h"
#include "SEGGER_RTT_Conf.h"
#include "nrf_drv_gpiote.h"

#define CENTRAL_LINK_COUNT              0                                           /**< Number of central links used by the application. When changing this number remember to adjust the RAM settings*/
#define PERIPHERAL_LINK_COUNT           1                                           /**< Number of peripheral links used by the application. When changing this number remember to adjust the RAM settings*/

#define ADVERTISING_LED_PIN             BSP_LED_0_MASK                              /**< Is on when device is advertising. */
#define CONNECTED_LED_PIN               BSP_LED_1_MASK                              /**< Is on when device has connected. */
#define ALARM_LED 											BSP_LED_3_MASK

#define BATTERY_UPDATE_LED_PIN               BSP_LED_2_MASK                              /**< LED to be toggled with the help of the LED Button Service. */

#define DEVICE_NAME                     "Demo BLE"                              /**< Name of device. Will be included in the advertising data. */

#define APP_ADV_INTERVAL                MSEC_TO_UNITS(100, UNIT_0_625_MS)           /**< The advertising interval (in units of 0.625 ms; this value corresponds to 40 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS      15             															/**< The advertising time-out (in units of seconds). When set to 0, we will never time out. */

#define APP_TIMER_PRESCALER             15                                          /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_MAX_TIMERS            6                                           /**< Maximum number of simultaneously created timers. */
#define APP_TIMER_OP_QUEUE_SIZE         6                                           /**< Size of timer operation queues. */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(500, UNIT_1_25_MS)            /**< Minimum acceptable connection interval (0.5 seconds). */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(800, UNIT_1_25_MS)            /**< Maximum acceptable connection interval (1 second). */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(6000, UNIT_10_MS)             /**< Connection supervisory time-out (4 seconds). */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(3000, APP_TIMER_PRESCALER)  /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (15 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000, APP_TIMER_PRESCALER)  /**< Time between each call to sd_ble_gap_conn_param_update after the first call (5 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                            /**< Number of attempts before giving up the connection parameter negotiation. */

#define APP_GPIOTE_MAX_USERS            1                                           /**< Maximum number of users of the GPIOTE handler. */
#define BUTTON_DETECTION_DELAY          APP_TIMER_TICKS(50, APP_TIMER_PRESCALER)    /**< Delay from a GPIOTE event until a button is reported as pushed (in number of timer ticks). */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define CHANNEL_RTT											0																						/** Channel pour le RTT Viewver**/
#define PIN_OUTPUT											12																					/** Pin qui va en entre du MOSFET pour la commande des vibreurs**/

static uint16_t                         m_conn_handle = BLE_CONN_HANDLE_INVALID;    /**< Handle of the current connection. */
static ble_lbs_t                        m_lbs;          														/**< LED Button Service instance. */

//	static uint8_t 		alarm;
static uint16_t	vibrationOn = 500, vibrationOff = 500, battery_update_time = 60000 /* in ms */;
static uint8_t battery_value = 100;
static volatile bool is_advertising = false;

APP_TIMER_DEF(timer_id_vibOn);
APP_TIMER_DEF(timer_id_vibOff);
APP_TIMER_DEF(timer_id_battery_update);
APP_TIMER_DEF(timer_id_alarm);


/*dclaration of app_button_handler(...) function*/
static void app_button_handler(uint8_t pin_no, uint8_t button_action);


/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze 
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


/**@brief Function for the LEDs initialization.
 *
 * @details Initializes all LEDs used by the application.
 */
static void leds_init(void)
{
    LEDS_CONFIGURE(ADVERTISING_LED_PIN | CONNECTED_LED_PIN | BATTERY_UPDATE_LED_PIN | ALARM_LED);
    LEDS_OFF(ADVERTISING_LED_PIN | CONNECTED_LED_PIN | BATTERY_UPDATE_LED_PIN | ALARM_LED);
}


/*** function alarm_vibration_on timer handler***/
static void alarm_vibration_on_handler(void* p_context) {
		
		LEDS_OFF(ALARM_LED);
		nrf_drv_gpiote_out_toggle(PIN_OUTPUT);
		app_timer_start(timer_id_vibOff, APP_TIMER_TICKS(vibrationOff, APP_TIMER_PRESCALER), NULL);
	
}

/*** function alarm_vibration_off timer handler***/
static void alarm_vibration_off_handler(void* p_context) {
	
		LEDS_ON(ALARM_LED);     
	  nrf_drv_gpiote_out_toggle(PIN_OUTPUT); // Pin 12 va en entre du transistor MOSFET de commande des vibreurs.
		app_timer_start(timer_id_vibOn, APP_TIMER_TICKS(vibrationOn, APP_TIMER_PRESCALER), NULL);
	
}


/*** function battery timer handler***/
static void battery_update_handler(void* p_context) {
	
		uint32_t err_code;
		ble_gatts_value_t gatts_value;
		memset(&gatts_value, 0, sizeof(gatts_value));
	
	  gatts_value.len = sizeof(uint8_t);
		gatts_value.offset  = 0;
    gatts_value.p_value = &battery_value;
	
		LEDS_INVERT(BATTERY_UPDATE_LED_PIN);
		battery_value--;
		if(battery_value == 30) battery_value = 100;
		err_code = sd_ble_gatts_value_set(BLE_CONN_HANDLE_INVALID, m_lbs.battery_char_handles.value_handle, &gatts_value);
		APP_ERROR_CHECK(err_code);
	
		SEGGER_RTT_printf(CHANNEL_RTT, "Battery value: %d\n", *(gatts_value.p_value) );
}



/*** function alarm timer handler***/

static void alarm_handler(void* p_context) {
	
		app_timer_start(timer_id_vibOn, APP_TIMER_TICKS(vibrationOn, APP_TIMER_PRESCALER), NULL);
		
}


/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module.
 */
static void timers_init(void)
{
    // Initialize timer module, making it use the scheduler
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);
		app_timer_create(&timer_id_vibOn, APP_TIMER_MODE_SINGLE_SHOT, alarm_vibration_on_handler);
		app_timer_create(&timer_id_vibOff, APP_TIMER_MODE_SINGLE_SHOT, alarm_vibration_off_handler);
	  app_timer_create(&timer_id_battery_update, APP_TIMER_MODE_REPEATED, battery_update_handler);
		app_timer_create(&timer_id_alarm, APP_TIMER_MODE_SINGLE_SHOT, alarm_handler);
	
		app_timer_start(timer_id_battery_update, APP_TIMER_TICKS(battery_update_time, APP_TIMER_PRESCALER), NULL);
}


/**@brief Function for the GAP initialization.
 *
 * @details This function sets up all the necessary GAP (Generic Access Profile) parameters of the
 *          device including the device name, appearance, and the preferred connection parameters.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *)DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;
																					
		int min_interval = gap_conn_params.min_conn_interval * 1.25;
		int max_interval = gap_conn_params.max_conn_interval * 1.25;
																					
		SEGGER_RTT_printf(CHANNEL_RTT, "MinConn: %d\nMaxConn: %d	\n", min_interval, max_interval);
		SEGGER_RTT_printf(CHANNEL_RTT, "SlaveLatency: %d \ntimeout: %d	\n", gap_conn_params.slave_latency, (gap_conn_params.conn_sup_timeout)*10 );																		

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for initializing the Advertising functionality.
 *
 * @details Encodes the required advertising data and passes it to the stack.
 *          Also builds a structure to be passed to the stack when starting advertising.
 */
static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
	
	
/*    ble_advdata_t scanrsp;

    ble_uuid_t adv_uuids[] = {{LBS_UUID_SERVICE, m_lbs.uuid_type}};
*/
	
    // Build and set advertising data
    memset(&advdata, 0, sizeof(advdata));

    advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance = true;
    advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;


/*    memset(&scanrsp, 0, sizeof(scanrsp));
    scanrsp.uuids_complete.uuid_cnt = sizeof(adv_uuids) / sizeof(adv_uuids[0]);
    scanrsp.uuids_complete.p_uuids  = adv_uuids;
*/
		
		err_code = sd_ble_gap_appearance_set(BLE_APPEARANCE_GENERIC_CLOCK);
		APP_ERROR_CHECK(err_code);
		
    err_code = ble_advdata_set(&advdata, NULL);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling write events to the LED characteristic.
 *
 * @param[in] p_lbs     Instance of LED Button Service to which the write applies.
 * @param[in] led_state Written/desired state of the LED.
 */
static void alarm_write_handler(ble_lbs_t * p_lbs, uint32_t alarm_in_ms)
{
		if(alarm_in_ms) {
			
			SEGGER_RTT_printf(CHANNEL_RTT, "led_write_handler() + %d", alarm_in_ms);
			app_timer_start(timer_id_alarm, APP_TIMER_TICKS(alarm_in_ms, APP_TIMER_PRESCALER), NULL);
		}
		
		else {
			
			app_timer_stop(timer_id_vibOn);
			app_timer_stop(timer_id_vibOff);
			LEDS_OFF(ALARM_LED);
			
		} 
		
}


/**@brief Function for handling write events to the VIBRATION_ON characteristic.
 *
 * @param[in] p_lbs     Instance of LED Button Service to which the write applies.
 * @param[in] led_state Written/desired state of the LED.
 */
static void vibrationOn_write_handler(ble_lbs_t * p_lbs, uint16_t vibOn_value)
{
	vibrationOn = vibOn_value;
	SEGGER_RTT_printf(CHANNEL_RTT, "valeur de vibrationOn %d\n", vibrationOn); 
	
}


/**@brief Function for handling write events to the VIBRATION_OFF characteristic.
 *
 * @param[in] p_lbs     Instance of LED Button Service to which the write applies.
 * @param[in] led_state Written/desired state of the LED.
 */
static void vibrationOff_write_handler(ble_lbs_t * p_lbs, uint16_t vibOff_value)
{
	
	vibrationOff = vibOff_value;
	SEGGER_RTT_printf(CHANNEL_RTT, "valeur de vibrationOff %d\n", vibrationOff); 
	
}



/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    uint32_t       err_code;
    ble_lbs_init_t init;

    init.alarm_write_handler = alarm_write_handler;
		init.vibrationOn_write_handler = vibrationOn_write_handler;
		init.vibrationOff_write_handler = vibrationOff_write_handler;

    err_code = ble_lbs_init(&m_lbs, &init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module that
 *          are passed to the application.
 *
 * @note All this function does is to disconnect. This could have been done by simply
 *       setting the disconnect_on_fail config parameter, but instead we use the event
 *       handler mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
				SEGGER_RTT_printf(CHANNEL_RTT, "BLE_CONN_PARAMS_EVT_FAILED \n");
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
			
    }
}


/**@brief Function for handling a Connection Parameters error.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init); 
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for starting advertising.
 */
static void advertising_start(void)
{
    uint32_t             err_code;
    ble_gap_adv_params_t adv_params;

    // Start advertising
    memset(&adv_params, 0, sizeof(adv_params));

    adv_params.type        = BLE_GAP_ADV_TYPE_ADV_IND;
    adv_params.p_peer_addr = NULL;
    adv_params.fp          = BLE_GAP_ADV_FP_ANY;
    adv_params.interval    = APP_ADV_INTERVAL;
    adv_params.timeout     = APP_ADV_TIMEOUT_IN_SECONDS;

    err_code = sd_ble_gap_adv_start(&adv_params);
    APP_ERROR_CHECK(err_code);
    LEDS_ON(ADVERTISING_LED_PIN); 
}


/**@brief Function for handling the Application's BLE stack events.
 *
 * @param[in] p_ble_evt  Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            LEDS_ON(CONNECTED_LED_PIN);
            LEDS_OFF(ADVERTISING_LED_PIN);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;

            break;

        case BLE_GAP_EVT_DISCONNECTED:
            LEDS_OFF(CONNECTED_LED_PIN);
            m_conn_handle = BLE_CONN_HANDLE_INVALID;

				    is_advertising = true;
            advertising_start();
            break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle,
                                                   BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP,
                                                   NULL,
                                                   NULL);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;
				
				case BLE_GAP_EVT_TIMEOUT:
					
						is_advertising = false;
						app_button_enable();
						LEDS_OFF(ADVERTISING_LED_PIN);
				    SEGGER_RTT_printf(CHANNEL_RTT, "Advertising: %s!\n", is_advertising ? "true" : "false"); 
						
						break;	

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for dispatching a BLE stack event to all modules with a BLE stack event handler.
 *
 * @details This function is called from the scheduler in the main loop after a BLE stack
 *          event has been received.
 *
 * @param[in] p_ble_evt  Bluetooth stack event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
    on_ble_evt(p_ble_evt);
    ble_conn_params_on_ble_evt(p_ble_evt);
    ble_lbs_on_ble_evt(&m_lbs, p_ble_evt);
}


/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    uint32_t err_code;
    
    nrf_clock_lf_cfg_t clock_lf_cfg = NRF_CLOCK_LFCLKSRC;
    
    // Initialize the SoftDevice handler module.
    SOFTDEVICE_HANDLER_INIT(&clock_lf_cfg, NULL);
    
    ble_enable_params_t ble_enable_params;
    err_code = softdevice_enable_get_default_config(CENTRAL_LINK_COUNT,
                                                    PERIPHERAL_LINK_COUNT,
                                                    &ble_enable_params);
    APP_ERROR_CHECK(err_code);
    
    //Check the ram settings against the used number of links
    CHECK_RAM_START_ADDR(CENTRAL_LINK_COUNT,PERIPHERAL_LINK_COUNT);
    
    // Enable BLE stack.
    err_code = softdevice_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);

    ble_gap_addr_t addr;

    err_code = sd_ble_gap_address_get(&addr);
    APP_ERROR_CHECK(err_code);
    err_code = sd_ble_gap_address_set(BLE_GAP_ADDR_CYCLE_MODE_NONE, &addr);
    APP_ERROR_CHECK(err_code);

    // Subscribe for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
}



/**@brief Function for the Power Manager.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();

    APP_ERROR_CHECK(err_code);
}

/***nrf_driver_gpiote initialization***/
static void gpiote_init(void) {
	
	// err_code.
	uint32_t err_code;
	
	// configuration de la pin 12.
	//nrf_drv_gpiote_pin_t pin = 12;
	nrf_drv_gpiote_out_config_t pin_config = GPIOTE_CONFIG_OUT_SIMPLE(false);
	
	
	if(!nrf_drv_gpiote_is_init()) {
		
		err_code = nrf_drv_gpiote_init();
		APP_ERROR_CHECK(err_code);
	}
	
	err_code = nrf_drv_gpiote_out_init(PIN_OUTPUT, &pin_config);
	APP_ERROR_CHECK(err_code);
	
}


/***app_button function initialization***/
static void button_init(void) {
	
	app_button_cfg_t button_config = {.pin_no = BUTTON_1, .active_state = APP_BUTTON_ACTIVE_HIGH,
																		.pull_cfg = NRF_GPIO_PIN_PULLUP, .button_handler = app_button_handler};
	
	uint32_t err_code;
	uint8_t button_count = 1;																	
	uint8_t detection_delay = 100;
																		
	err_code = app_button_init(&button_config, button_count, detection_delay); // 
	APP_ERROR_CHECK(err_code);
																		
	err_code = app_button_enable();																	
	APP_ERROR_CHECK(err_code);
																		
}

/***button_handler***/
static void app_button_handler(uint8_t pin_no, uint8_t button_action) {

	if(!is_advertising) {
		
		is_advertising = true;
		SEGGER_RTT_printf(CHANNEL_RTT, "Advertising: %s!\n", is_advertising ? "true" : "false");
		app_button_disable();
		advertising_start();
	}
	
	
}

/**@brief Function for application main entry.
 */
int main(void)
{
	
	  SEGGER_RTT_WriteString(CHANNEL_RTT, "main function !\n");
	
    // Initialize.
    leds_init();
    timers_init();
		gpiote_init();
    ble_stack_init();
    gap_params_init();
    services_init();
    advertising_init();
    conn_params_init();
		button_init();

    // Start execution.
	
    // advertising_start(); 
	
    // SEGGER_RTT_WriteString(0, "Advertising start !\n");
	
    // Enter main loop.
    for (;;)
    {
        power_manage();
			
    }
}


/**
 * @}
 */
